#
#
#
#
#
#

import time, datetime
import opcua
import requests
import configparser

# 读取配置文件
config = configparser.ConfigParser()
config.read('config.ini', encoding='utf8')

# Http请求头
headers = {'Content-Type': 'application/json;charset=utf-8'}
# OpcSvr地址
opcUrl = config['CLConfig']['url']
# 机器人地址
webhook = config['CLConfig']['webhook']
# @用户列表
at_user = config['CLConfig']['at_user']
# @用户不为空时拼接
if at_user:
    at_user = at_user + '\n'
# 线别信息
lines = config['CLConfig']['lineList']
# 监控节点信息
nodes = config['CLConfig']['nodeList']
# 循环间隔时间
cycleSeconds = config['CLConfig']['seconds']
# 初始化list
lineList = lines.split(',')
NameList = nodes.split(',')

# OpcNodeID列表
nodeIdlist = []
# OpcNodeID列表读取值list
node_valuesNow = []
# nodeIdlist = []
# node_valuesNow=[]
# OpcNodeID节点对应别名list
nodeNameList = []
# get_nodes = {}
# OPCNodeID节点原始比较值list
node_valuesOrign = []

# 遍历拼接节点别名list和NodeID节点list
for lineName in lineList:
    for nodeName in NameList:
        # 拼接节点名称
        str1 = ''.join([lineName, '.', nodeName])
        # 拼接OpcNodeID节点
        str2 = ''.join(['ns=2;s=M1Press.', str1])
        # OpcNodeID节点List,Node节点Value初始值
        nodeIdlist.append(str2)
        # node_valuesOrign.append(False)
        # node_valuesNow.append(False)
        # 节点名称字典、节点值字典
        nodeNameList.append(str1)
# print(nodeIdlist)
# 初始化list长度
node_valuesOrign = [False] * len(nodeIdlist)
node_valuesNow = [False] * len(nodeIdlist)


# OPCUA标签订阅功能
class SubHandler(object):
    """
    Subscription Handler. To receive events from server for a subscription
    data_change and event methods are called directly from receiving thread.
    Do not do expensive, slow or network operation there. Create another
    thread if you need to do such a thing
    """

    # 当有数据发生变化时会执行该方法
    def datachange_notification(self, node, val, data):
        node_valuesNow[nodeIdlist.index(str(node))] = val
        print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), " DataChangeEvent: ", node, val)

    def event_notification(self, event):
        print("Python: New event", event)


# 拼接报警文本消息
def createMsg(content):
    title = '# 告警信息 <font color=\"warning\">层压机</font>\n'
    time = '> <font color=\"info\">发生时间：' + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S") + '</font>\n'
    msgHeader = '{\\"msgtype\\":\\"markdown\\",\\"markdown\\":{\"content\":\"'
    msgEnd = '\"}}'
    data = {
        "msgtype": "markdown",
        "markdown": {
            "content": title + content + time + at_user
        }
    }
    # return msgHeader + title + content + time + at_user + msgEnd
    return data


# OPCUA连接
client = opcua.Client(opcUrl)
client.set_user('mes')
client.set_password('mesmes')
client.connect()

# 初始化OPCUA，GetNode列表
get_nodesList = []
for item in nodeIdlist:
    node = client.get_node(item)
    get_nodesList.append(node)

handler = SubHandler()
# 创建订阅
sub = client.create_subscription(500, handler)
# print(sub.is_ready())


# 数据变化时，可监视多个节点组成的列表
handle = sub.subscribe_data_change(get_nodesList)
time.sleep(5)

# 功能体
n=0
while True:
    # 线程睡眠时间
    time.sleep(int(cycleSeconds))
    # print(len(node_valuesNow))
    # 读取值和原始值比较判断，变化时执行
    if not (node_valuesOrign == node_valuesNow):
        # 信息文本
        content = ''
        for i in range(len(node_valuesNow)):
            # 有过故障执行
            if node_valuesOrign[i] or node_valuesNow[i]:
                # 故障新增
                if node_valuesNow[i]:
                    content += f'> <font color=\"warning\">故障站点：{nodeNameList[i]}</font>\n'
                # 故障解除
                else:
                    content += f'> <font color=\"info\">故障消除：{nodeNameList[i]}</font>\n'
        # 读取值赋给原始值
        node_valuesOrign = node_valuesNow[0:]
        msg = createMsg(content)
        r = requests.post(webhook, headers=headers, json=msg)
        print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), r.content)
        n += 1
        if n % 100 == 0:
            print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), f'循环读取第{n}次')

        # print(node_valuesNow)
        # print('-'*50)
        # print(node_valuesOrign==node_valuesNow)
        # print(node_valuesOrign)

# 取消handle的订阅，执行后数据变化时不会再执行handle
# sub.unsubscribe(handle)

# 删除服务器上的订阅。这是由客户端和服务器类在退出时自动完成的
# sub.delete()

# client.disconnect()
